@@ -123,11 +123,11 @@ public final class DateTime
     /**
      * Parses a {@code DateTime} from the specified string.
      * <p>
-     * This uses {@link ISODateTimeFormat#dateTimeParser().withOffsetParsed()}
+     * This uses {@link ISODateTimeFormat#dateTimeParser()}{@code .withOffsetParsed()}
      * which is different to passing a {@code String} to the constructor.
      * <p>
      * Sometimes this method and {@code new DateTime(str)} return different results.
-     * This can be confusing as the different is not visible in {@link #toString()}.
+     * This can be confusing as the difference is not visible in {@link #toString()}.
      * <p>
      * When passed a date-time string without an offset, such as '2010-06-30T01:20',
      * both the constructor and this method use the default time-zone.
@@ -138,7 +138,7 @@ public final class DateTime
      * the offset is directly parsed and stored.
      * As such, {@code DateTime.parse("2010-06-30T01:20+02:00")} and
      * {@code new DateTime("2010-06-30T01:20+02:00"))} are NOT equal.
-     * The object produced via this method has a zone of {@code DateTimeZone.forOffsetHours(1)}.
+     * The object produced via this method has a zone of {@code DateTimeZone.forOffsetHours(2)}.
      * The object produced via the constructor has a zone of {@code DateTimeZone.getDefault()}.
      * 
      * @param str  the string to parse, not null
@@ -713,6 +713,8 @@ public final class DateTime
      * DateTime set = monthOfYear().setCopy(6);
      * </pre>
      * <p>
+     * If the time is invalid on the new date due to the time-zone, the time will be adjusted.
+     * <p>
      * This instance is immutable and unaffected by this method call.
      *
      * @param year  the new year value
@@ -723,11 +725,25 @@ public final class DateTime
      */
     public DateTime withDate(int year, int monthOfYear, int dayOfMonth) {
         Chronology chrono = getChronology();
-        long instant = getMillis();
-        instant = chrono.year().set(instant, year);
-        instant = chrono.monthOfYear().set(instant, monthOfYear);
-        instant = chrono.dayOfMonth().set(instant, dayOfMonth);
-        return withMillis(instant);
+        long localInstant = chrono.withUTC().getDateTimeMillis(year, monthOfYear, dayOfMonth, getMillisOfDay());
+        return withMillis(chrono.getZone().convertLocalToUTC(localInstant, false, getMillis()));
+    }
+
+    /**
+     * Returns a copy of this datetime with the specified date, retaining the time fields.
+     * <p>
+     * If the time is invalid on the new date due to the time-zone, the time will be adjusted.
+     * <p>
+     * This instance is immutable and unaffected by this method call.
+     *
+     * @param date  the local date
+     * @return a copy of this datetime with a different date
+     * @throws IllegalArgumentException if the time-of-day is invalid for the date
+     * @throws NullPointerException if the date is null
+     */
+    public DateTime withDate(LocalDate date) {
+        return withDate(
+            date.getYear(), date.getMonthOfYear(), date.getDayOfMonth());
     }
 
     /**
@@ -740,6 +756,8 @@ public final class DateTime
      * DateTime set = dt.hourOfDay().setCopy(6);
      * </pre>
      * <p>
+     * If the new time is invalid due to the time-zone, the time will be adjusted.
+     * <p>
      * This instance is immutable and unaffected by this method call.
      *
      * @param hourOfDay  the hour of the day
@@ -751,12 +769,26 @@ public final class DateTime
      */
     public DateTime withTime(int hourOfDay, int minuteOfHour, int secondOfMinute, int millisOfSecond) {
         Chronology chrono = getChronology();
-        long instant = getMillis();
-        instant = chrono.hourOfDay().set(instant, hourOfDay);
-        instant = chrono.minuteOfHour().set(instant, minuteOfHour);
-        instant = chrono.secondOfMinute().set(instant, secondOfMinute);
-        instant = chrono.millisOfSecond().set(instant, millisOfSecond);
-        return withMillis(instant);
+        long localInstant = chrono.withUTC().getDateTimeMillis(
+            getYear(), getMonthOfYear(), getDayOfMonth(), hourOfDay, minuteOfHour, secondOfMinute, millisOfSecond);
+        return withMillis(chrono.getZone().convertLocalToUTC(localInstant, false, getMillis()));
+    }
+
+    /**
+     * Returns a copy of this datetime with the specified time, retaining the date fields.
+     * <p>
+     * If the new time is invalid due to the time-zone, the time will be adjusted.
+     * <p>
+     * This instance is immutable and unaffected by this method call.
+     *
+     * @param time  the local time
+     * @return a copy of this datetime with a different time
+     * @throws IllegalArgumentException if the time-of-day is invalid for the date
+     * @throws NullPointerException if the time is null
+     */
+    public DateTime withTime(LocalTime time) {
+        return withTime(
+            time.getHourOfDay(), time.getMinuteOfHour(), time.getSecondOfMinute(), time.getMillisOfSecond());
     }
 
     /**
