@@ -360,11 +360,9 @@ public class DateTimeZoneBuilder {
             rs = new RuleSet(rs);
 
             while ((next = rs.nextTransition(millis, saveMillis)) != null) {
-                if (addTransition(transitions, next)) {
-                    if (tailZone != null) {
-                        // Got the extra transition before DSTZone.
-                        break;
-                    }
+                if (addTransition(transitions, next) && tailZone != null) {
+                    // Got the extra transition before DSTZone.
+                    break;
                 }
                 millis = next.getMillis();
                 saveMillis = next.getSaveMillis();
@@ -443,7 +441,9 @@ public class DateTimeZoneBuilder {
         if (out instanceof DataOutput) {
             writeTo(zoneID, (DataOutput)out);
         } else {
-            writeTo(zoneID, (DataOutput)new DataOutputStream(out));
+            DataOutputStream dout = new DataOutputStream(out);
+            writeTo(zoneID, (DataOutput)dout);
+            dout.flush();
         }
     }
 
@@ -1443,7 +1443,7 @@ public class DateTimeZoneBuilder {
                         curNameKey.equals(zoneNameData[2]) &&
                         curNameKey.equals(zoneNameData[4])) {
                     
-                    if (ZoneInfoCompiler.verbose()) {
+                    if (ZoneInfoLogger.verbose()) {
                         System.out.println("Fixing duplicate name key - " + nextNameKey);
                         System.out.println("     - " + new DateTime(trans[i], chrono) +
                                            " - " + new DateTime(trans[i + 1], chrono));
@@ -1460,7 +1460,7 @@ public class DateTimeZoneBuilder {
             if (tailZone != null) {
                 if (tailZone.iStartRecurrence.getNameKey()
                     .equals(tailZone.iEndRecurrence.getNameKey())) {
-                    if (ZoneInfoCompiler.verbose()) {
+                    if (ZoneInfoLogger.verbose()) {
                         System.out.println("Fixing duplicate recurrent name key - " +
                                            tailZone.iStartRecurrence.getNameKey());
                     }
