@@ -29,9 +29,9 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
+import java.util.Map.Entry;
 import java.util.StringTokenizer;
 import java.util.TreeMap;
-import java.util.Map.Entry;
 
 import org.joda.time.Chronology;
 import org.joda.time.DateTime;
@@ -65,20 +65,6 @@ public class ZoneInfoCompiler {
 
     static Chronology cLenientISO;
 
-    static ThreadLocal<Boolean> cVerbose = new ThreadLocal<Boolean>() {
-        protected Boolean initialValue() {
-            return Boolean.FALSE;
-        }
-    };
-
-    /**
-     * Gets a flag indicating that verbose logging is required.
-     * @return true to log verbosely
-     */
-    public static boolean verbose() {
-        return cVerbose.get();
-    }
-
     //-----------------------------------------------------------------------
     /**
      * Launches the ZoneInfoCompiler tool.
@@ -132,7 +118,7 @@ public class ZoneInfoCompiler {
             sources[j] = inputDir == null ? new File(args[i]) : new File(inputDir, args[i]);
         }
 
-        cVerbose.set(verbose);
+        ZoneInfoLogger.set(verbose);
         ZoneInfoCompiler zic = new ZoneInfoCompiler();
         zic.compile(outputDir, sources);
     }
@@ -350,12 +336,16 @@ public class ZoneInfoCompiler {
     private List<Zone> iZones;
 
     // List String pairs to link.
-    private List<String> iLinks;
+    private List<String> iGoodLinks;
+
+    // List String pairs to link.
+    private List<String> iBackLinks;
 
     public ZoneInfoCompiler() {
         iRuleSets = new HashMap<String, RuleSet>();
         iZones = new ArrayList<Zone>();
-        iLinks = new ArrayList<String>();
+        iGoodLinks = new ArrayList<String>();
+        iBackLinks = new ArrayList<String>();
     }
 
     /**
@@ -368,16 +358,14 @@ public class ZoneInfoCompiler {
         if (sources != null) {
             for (int i=0; i<sources.length; i++) {
                 BufferedReader in = new BufferedReader(new FileReader(sources[i]));
-                parseDataFile(in);
+                parseDataFile(in, "backward".equals(sources[i].getName()));
                 in.close();
             }
         }
 
         if (outputDir != null) {
-            if (!outputDir.exists()) {
-                if (!outputDir.mkdirs()) {
-                    throw new IOException("Destination directory doesn't exist and cannot be created: " + outputDir);
-                }
+            if (!outputDir.exists() && !outputDir.mkdirs()) {
+                throw new IOException("Destination directory doesn't exist and cannot be created: " + outputDir);
             }
             if (!outputDir.isDirectory()) {
                 throw new IOException("Destination is not a directory: " + outputDir);
@@ -385,60 +373,68 @@ public class ZoneInfoCompiler {
         }
 
         Map<String, DateTimeZone> map = new TreeMap<String, DateTimeZone>();
+        Map<String, Zone> sourceMap = new TreeMap<String, Zone>();
 
         System.out.println("Writing zoneinfo files");
-        for (int i=0; i<iZones.size(); i++) {
+        // write out the standard entries
+        for (int i = 0; i < iZones.size(); i++) {
             Zone zone = iZones.get(i);
             DateTimeZoneBuilder builder = new DateTimeZoneBuilder();
             zone.addToBuilder(builder, iRuleSets);
-            final DateTimeZone original = builder.toDateTimeZone(zone.iName, true);
-            DateTimeZone tz = original;
+            DateTimeZone tz = builder.toDateTimeZone(zone.iName, true);
             if (test(tz.getID(), tz)) {
                 map.put(tz.getID(), tz);
+                sourceMap.put(tz.getID(), zone);
                 if (outputDir != null) {
-                    if (ZoneInfoCompiler.verbose()) {
-                        System.out.println("Writing " + tz.getID());
-                    }
-                    File file = new File(outputDir, tz.getID());
-                    if (!file.getParentFile().exists()) {
-                        file.getParentFile().mkdirs();
-                    }
-                    OutputStream out = new FileOutputStream(file);
-                    try {
-                        builder.writeTo(zone.iName, out);
-                    } finally {
-                        out.close();
-                    }
-
-                    // Test if it can be read back.
-                    InputStream in = new FileInputStream(file);
-                    DateTimeZone tz2 = DateTimeZoneBuilder.readFrom(in, tz.getID());
-                    in.close();
-
-                    if (!original.equals(tz2)) {
-                        System.out.println("*e* Error in " + tz.getID() +
-                                           ": Didn't read properly from file");
-                    }
+                    writeZone(outputDir, builder, tz);
                 }
             }
         }
 
-        for (int pass=0; pass<2; pass++) {
-            for (int i=0; i<iLinks.size(); i += 2) {
-                String id = iLinks.get(i);
-                String alias = iLinks.get(i + 1);
+        // revive zones from "good" links
+        for (int i = 0; i < iGoodLinks.size(); i += 2) {
+            String baseId = iGoodLinks.get(i);
+            String alias = iGoodLinks.get(i + 1);
+            Zone sourceZone = sourceMap.get(baseId);
+            if (sourceZone == null) {
+                System.out.println("Cannot find source zone '" + baseId + "' to link alias '" + alias + "' to");
+            } else {
+                DateTimeZoneBuilder builder = new DateTimeZoneBuilder();
+                sourceZone.addToBuilder(builder, iRuleSets);
+                DateTimeZone revived = builder.toDateTimeZone(alias, true);
+                if (test(revived.getID(), revived)) {
+                    map.put(revived.getID(), revived);
+                    if (outputDir != null) {
+                        writeZone(outputDir, builder, revived);
+                    }
+                }
+                map.put(revived.getID(), revived);
+                if (ZoneInfoLogger.verbose()) {
+                    System.out.println("Good link: " + alias + " -> " + baseId + " revived");
+                }
+            }
+        }
+
+        // store "back" links as aliases (where name is permanently mapped
+        for (int pass = 0; pass < 2; pass++) {
+            for (int i = 0; i < iBackLinks.size(); i += 2) {
+                String id = iBackLinks.get(i);
+                String alias = iBackLinks.get(i + 1);
                 DateTimeZone tz = map.get(id);
                 if (tz == null) {
                     if (pass > 0) {
-                        System.out.println("Cannot find time zone '" + id +
-                                           "' to link alias '" + alias + "' to");
+                        System.out.println("Cannot find time zone '" + id + "' to link alias '" + alias + "' to");
                     }
                 } else {
                     map.put(alias, tz);
+                    if (ZoneInfoLogger.verbose()) {
+                        System.out.println("Back link: " + alias + " -> " + tz.getID());
+                    }
                 }
             }
         }
 
+        // write map that unites the time-zone data, pointing aliases and real zones at files
         if (outputDir != null) {
             System.out.println("Writing ZoneInfoMap");
             File file = new File(outputDir, "ZoneInfoMap");
@@ -461,7 +457,33 @@ public class ZoneInfoCompiler {
         return map;
     }
 
-    public void parseDataFile(BufferedReader in) throws IOException {
+    private void writeZone(File outputDir, DateTimeZoneBuilder builder, DateTimeZone tz) throws IOException {
+        if (ZoneInfoLogger.verbose()) {
+            System.out.println("Writing " + tz.getID());
+        }
+        File file = new File(outputDir, tz.getID());
+        if (!file.getParentFile().exists()) {
+            file.getParentFile().mkdirs();
+        }
+        OutputStream out = new FileOutputStream(file);
+        try {
+            builder.writeTo(tz.getID(), out);
+        } finally {
+            out.close();
+        }
+
+        // Test if it can be read back.
+        InputStream in = new FileInputStream(file);
+        DateTimeZone tz2 = DateTimeZoneBuilder.readFrom(in, tz.getID());
+        in.close();
+
+        if (!tz.equals(tz2)) {
+            System.out.println("*e* Error in " + tz.getID() +
+                               ": Didn't read properly from file");
+        }
+    }
+
+    public void parseDataFile(BufferedReader in, boolean backward) throws IOException {
         Zone zone = null;
         String line;
         while ((line = in.readLine()) != null) {
@@ -504,10 +526,23 @@ public class ZoneInfoCompiler {
                         rs.addRule(r);
                     }
                 } else if (token.equalsIgnoreCase("Zone")) {
+                    if (st.countTokens() < 4) {
+                        throw new IllegalArgumentException("Attempting to create a Zone from an incomplete tokenizer");
+                    }
                     zone = new Zone(st);
                 } else if (token.equalsIgnoreCase("Link")) {
-                    iLinks.add(st.nextToken());
-                    iLinks.add(st.nextToken());
+                    String real = st.nextToken();
+                    String alias = st.nextToken();
+                    // links in "backward" are deprecated names
+                    // links in other files should be kept
+                    // special case a few to try to repair terrible damage to tzdb
+                    if (backward || alias.equals("US/Pacific-New") || alias.startsWith("Etc/") || alias.equals("GMT")) {
+                        iBackLinks.add(real);
+                        iBackLinks.add(alias);
+                    } else {
+                        iGoodLinks.add(real);
+                        iGoodLinks.add(alias);
+                    }
                 } else {
                     System.out.println("Unknown line: " + line);
                 }
@@ -581,13 +616,20 @@ public class ZoneInfoCompiler {
                         str = st.nextToken();
                         zoneChar = parseZoneChar(str.charAt(str.length() - 1));
                         if (str.equals("24:00")) {
-                            LocalDate date = (day == -1 ?
-                                    new LocalDate(2001, month, 1).plusMonths(1) :
-                                    new LocalDate(2001, month, day).plusDays(1));
-                            advance = (day != -1);
-                            month = date.getMonthOfYear();
-                            day = date.getDayOfMonth();
-                            dayOfWeek = ((dayOfWeek - 1 + 1) % 7) + 1;
+                            // handle end of year
+                            if (month == 12 && day == 31) {
+                                millis = parseTime("23:59:59.999");
+                            } else {
+                                LocalDate date = (day == -1 ?
+                                        new LocalDate(2001, month, 1).plusMonths(1) :
+                                        new LocalDate(2001, month, day).plusDays(1));
+                                advance = (day != -1 && dayOfWeek != 0);
+                                month = date.getMonthOfYear();
+                                day = date.getDayOfMonth();
+                                if (dayOfWeek != 0) {
+                                    dayOfWeek = ((dayOfWeek - 1 + 1) % 7) + 1;
+                                }
+                            }
                         } else {
                             millis = parseTime(str);
                         }
@@ -653,6 +695,9 @@ public class ZoneInfoCompiler {
         public final String iLetterS;
 
         Rule(StringTokenizer st) {
+            if (st.countTokens() < 6) {
+                throw new IllegalArgumentException("Attempting to create a Rule from an incomplete tokenizer");
+            }
             iName = st.nextToken().intern();
             iFromYear = parseYear(st.nextToken(), 0);
             iToYear = parseYear(st.nextToken(), iFromYear);
