@@ -19,7 +19,6 @@ import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
-import java.io.PrintStream;
 import java.lang.reflect.Modifier;
 import java.security.AllPermission;
 import java.security.CodeSource;
@@ -281,12 +280,14 @@ public class TestDateTimeZone extends TestCase {
             String value = map.get(key);
             TimeZone juZone = TimeZone.getTimeZone(key);
             DateTimeZone zone = DateTimeZone.forTimeZone(juZone);
-            assertEquals(value, zone.getID());
+            assertEquals(DateTimeZone.forID(value), zone);
 //            System.out.println(juZone);
 //            System.out.println(juZone.getDisplayName());
 //            System.out.println(zone);
 //            System.out.println("------");
         }
+        // gee thanks time-zone db maintainer for damaging the database
+        // and breaking the long-standing  association with CAT/EAT
     }
 
     //-----------------------------------------------------------------------
@@ -494,6 +495,21 @@ public class TestDateTimeZone extends TestCase {
             DateTimeZone.setProvider(null);
             assertEquals(ZoneInfoProvider.class, DateTimeZone.getProvider().getClass());
         }
+        
+        try {
+            System.setProperty("org.joda.time.DateTimeZone.Folder", "src/test/resources/tzdata");
+            DateTimeZone.setProvider(null);
+            assertEquals(ZoneInfoProvider.class, DateTimeZone.getProvider().getClass());
+            assertEquals(2, DateTimeZone.getAvailableIDs().size());
+            assertEquals(true, DateTimeZone.getAvailableIDs().contains("UTC"));
+            assertEquals(true, DateTimeZone.getAvailableIDs().contains("CET"));
+            
+        } finally {
+            System.getProperties().remove("org.joda.time.DateTimeZone.Folder");
+            DateTimeZone.setProvider(null);
+            assertEquals(ZoneInfoProvider.class, DateTimeZone.getProvider().getClass());
+            assertEquals(true, DateTimeZone.getAvailableIDs().size() > 2);
+        }
     }
 
     public void testProvider_badClassName() {
@@ -1009,8 +1025,8 @@ public class TestDateTimeZone extends TestCase {
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
         ObjectOutputStream oos = new ObjectOutputStream(baos);
         oos.writeObject(zone);
-        byte[] bytes = baos.toByteArray();
         oos.close();
+        byte[] bytes = baos.toByteArray();
         
         ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
         ObjectInputStream ois = new ObjectInputStream(bais);
@@ -1027,15 +1043,15 @@ public class TestDateTimeZone extends TestCase {
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
         ObjectOutputStream oos = new ObjectOutputStream(baos);
         oos.writeObject(zone);
-        byte[] bytes = baos.toByteArray();
         oos.close();
+        byte[] bytes = baos.toByteArray();
         
         ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
         ObjectInputStream ois = new ObjectInputStream(bais);
         DateTimeZone result = (DateTimeZone) ois.readObject();
         ois.close();
         
-        assertSame(zone, result);
+        assertEquals(zone, result);
     }
 
     public void testCommentParse() throws Exception {
