@@ -487,8 +487,8 @@ public class TestDateTime_Basics extends TestCase {
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
         ObjectOutputStream oos = new ObjectOutputStream(baos);
         oos.writeObject(test);
-        byte[] bytes = baos.toByteArray();
         oos.close();
+        byte[] bytes = baos.toByteArray();
         
         ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
         ObjectInputStream ois = new ObjectInputStream(bais);
@@ -839,7 +839,45 @@ public class TestDateTime_Basics extends TestCase {
         } catch (IllegalArgumentException ex) {}
     }
     
-    public void testWithTime_int_int_int() {
+    public void testWithDate_int_int_int_toDST1() {
+        // 2010-03-28T02:55 is DST time, need to change to 03:55
+        DateTime test = new DateTime(2015, 1, 10, 2, 55, 0, 0, ISO_PARIS);
+        DateTime result = test.withDate(2010, 3, 28);
+        DateTime expected = new DateTime(2010, 3, 28, 3, 55, 0, 0, ISO_PARIS);
+        assertEquals(expected, result);
+    }
+    
+    public void testWithDate_int_int_int_toDST2() {
+        // 2010-03-28T02:55 is DST time, need to change to 03:55
+        DateTime test = new DateTime(2015, 1, 28, 2, 55, 0, 0, ISO_PARIS);
+        DateTime result = test.withDate(2010, 3, 28);
+        DateTime expected = new DateTime(2010, 3, 28, 3, 55, 0, 0, ISO_PARIS);
+        assertEquals(expected, result);
+    }
+    
+    public void testWithDate_int_int_int_affectedByDST() {
+        // 2010-03-28T02:55 is DST time, need to avoid time being changed to 03:55
+        DateTime test = new DateTime(2015, 1, 28, 2, 55, 0, 0, ISO_PARIS);
+        DateTime result = test.withDate(2010, 3, 10);
+        DateTime expected = new DateTime(2010, 3, 10, 2, 55, 0, 0, ISO_PARIS);
+        assertEquals(expected, result);
+    }
+    
+    public void testWithDate_LocalDate() {
+        DateTime test = new DateTime(2002, 4, 5, 1, 2, 3, 4, ISO_UTC);
+        DateTime result = test.withDate(new LocalDate(2003, 5, 6));
+        DateTime expected = new DateTime(2003, 5, 6, 1, 2, 3, 4, ISO_UTC);
+        assertEquals(expected, result);
+        
+        test = new DateTime(TEST_TIME1);
+        try {
+            test.withDate(new LocalDate(2003, 13, 1));
+            fail();
+        } catch (IllegalArgumentException ex) {}
+    }
+    
+    //-----------------------------------------------------------------------
+    public void testWithTime_int_int_int_int() {
         DateTime test = new DateTime(TEST_TIME1 - 12345L, BUDDHIST_UTC);
         DateTime result = test.withTime(12, 24, 0, 0);
         assertEquals(TEST_TIME1, result.getMillis());
@@ -852,6 +890,27 @@ public class TestDateTime_Basics extends TestCase {
         } catch (IllegalArgumentException ex) {}
     }
     
+    public void testWithTime_int_int_int_int_toDST() {
+        // 2010-03-28T02:55 is DST time, need to change to 03:55
+        DateTime test = new DateTime(2010, 3, 28, 0, 0, 0, 0, ISO_PARIS);
+        DateTime result = test.withTime(2, 55, 0, 0);
+        DateTime expected = new DateTime(2010, 3, 28, 3, 55, 0, 0, ISO_PARIS);
+        assertEquals(expected, result);
+    }
+    
+    public void testWithTime_LocalTime() {
+        DateTime test = new DateTime(TEST_TIME1 - 12345L, BUDDHIST_UTC);
+        DateTime result = test.withTime(new LocalTime(12, 24, 0, 0));
+        assertEquals(TEST_TIME1, result.getMillis());
+        assertEquals(BUDDHIST_UTC, result.getChronology());
+        
+        test = new DateTime(TEST_TIME1);
+        try {
+            test.withTime(new LocalTime(25, 1, 1, 1));
+            fail();
+        } catch (IllegalArgumentException ex) {}
+    }
+    
     @SuppressWarnings("deprecation")
     public void testWithFields_RPartial() {
         DateTime test = new DateTime(2004, 5, 6, 7, 8, 9, 0);
